require 'spec_helper'

describe Daedal::Queries::SimpleQueryStringQuery do

  subject do
    Daedal::Queries::SimpleQueryStringQuery
  end

  let(:hash_query) do
    {simple_query_string: {query: 'foo'}}
  end

  context 'without a query string' do
    it 'will raise an error' do
      expect{subject.new}.to raise_error(Virtus::CoercionError)
    end
  end

  context 'with a query that is not a string given' do
    let(:query) do
      subject.new query: :foo
    end
    it 'will convert to a string' do
      expect(query.query).to eq 'foo'
    end
  end

  context 'with a query' do
    let(:query) do
      subject.new query: 'foo'
    end
    it 'will create a query string with the correct value' do
      expect(query.query).to eq 'foo'
    end
    it 'will have the correct hash and json representation' do
      expect(query.to_hash).to eq hash_query
      expect(query.to_json).to eq hash_query.to_json
    end
  end

  context 'with a default_field' do
    let(:query) do
      subject.new query: 'foo', default_field: :bar
    end
    before do
      hash_query[:simple_query_string][:default_field] = :bar
    end
    it 'will set the default_field' do
      expect(query.default_field).to eq :bar
    end
    it 'will have the correct hash and json representations' do
      expect(query.to_hash).to eq hash_query
      expect(query.to_json).to eq hash_query.to_json
    end
  end

  context 'with an array of fields' do
    let(:query) do
      subject.new query: 'foo', fields: [:bar]
    end
    before do
      hash_query[:simple_query_string][:fields] = [:bar]
    end
    it 'will set the fields' do
      expect(query.fields).to eq [:bar]
    end
    it 'will have the correct hash and json representations' do
      expect(query.to_hash).to eq hash_query
      expect(query.to_json).to eq hash_query.to_json
    end
  end

  context 'with a default_operator' do
    let(:query) do
      subject.new query: 'foo', default_operator: 'OR'
    end
    before do
      hash_query[:simple_query_string][:default_operator] = 'OR'
    end
    it 'will set the default_operator' do
      expect(query.default_operator).to eq 'OR'
    end
    it 'will have the correct hash and json representations' do
      expect(query.to_hash).to eq hash_query
      expect(query.to_json).to eq hash_query.to_json
    end
  end

  context 'with an analyzer' do
    let(:query) do
      subject.new query: 'foo', analyzer: :bar
    end
    before do
      hash_query[:simple_query_string][:analyzer] = :bar
    end
    it 'will set the analyzer' do
      expect(query.analyzer).to eq :bar
    end
    it 'will have the correct hash and json representations' do
      expect(query.to_hash).to eq hash_query
      expect(query.to_json).to eq hash_query.to_json
    end
  end

  context 'with flags' do
    let(:query) do
      subject.new query: 'foo', flags: [:or, :and, :prefix]
    end
    before do
      hash_query[:simple_query_string][:flags] = 'or|and|prefix'
    end
    it 'will set the flags' do
      expect(query.flags).to eq [:or, :and, :prefix]
    end
    it 'will have the correct hash and json representations' do
      expect(query.to_hash).to eq hash_query
      expect(query.to_json).to eq hash_query.to_json
    end
  end

  context 'with a lowercase_expanded_terms' do
    let(:query) do
      subject.new query: 'foo', lowercase_expanded_terms: true
    end
    before do
      hash_query[:simple_query_string][:lowercase_expanded_terms] = true
    end
    it 'will set the lowercase_expanded_terms' do
      expect(query.lowercase_expanded_terms).to eq true
    end
    it 'will have the correct hash and json representations' do
      expect(query.to_hash).to eq hash_query
      expect(query.to_json).to eq hash_query.to_json
    end
  end

  context 'with a minimum_should_match' do
    let(:query) do
      subject.new query: 'foo', minimum_should_match: 2
    end
    before do
      hash_query[:simple_query_string][:minimum_should_match] = 2
    end
    it 'will set the minimum_should_match' do
      expect(query.minimum_should_match).to eq 2
    end
    it 'will have the correct hash and json representations' do
      expect(query.to_hash).to eq hash_query
      expect(query.to_json).to eq hash_query.to_json
    end
  end

  context 'with a lenient' do
    let(:query) do
      subject.new query: 'foo', lenient: true
    end
    before do
      hash_query[:simple_query_string][:lenient] = true
    end
    it 'will set the lenient' do
      expect(query.lenient).to eq true
    end
    it 'will have the correct hash and json representations' do
      expect(query.to_hash).to eq hash_query
      expect(query.to_json).to eq hash_query.to_json
    end
  end
end
